@@ -521,6 +521,22 @@ impl Section {
         self
     }
 
+    /// Append `section`'s contents to the end of this section.
+    ///
+    /// Any `Label`s that were appended to `section` will not be
+    /// resolved until this section is finalized.
+    /// Return this section.
+    pub fn append_section<S: Into<Section>>(mut self, section: S) -> Section {
+        let Section { contents, references, .. } = section.into();
+        let current = self.size();
+        self.contents.write_all(&contents.into_inner()).unwrap();
+        self.references.extend(references.into_iter().map(|mut r| {
+            r.offset = r.offset + current;
+            r
+        }));
+        self
+    }
+
     /// Append `count` copies of `byte` to the end of this section.
     ///
     /// Return this section.
@@ -907,6 +923,34 @@ fn section_append_bytes() {
                &[0, 1, 2, 3, 4, 0xf, 0xe, 0xd, 0xc, 0xb]);
 }
 
+#[test]
+fn section_append_section_simple() {
+    assert_eq!(Section::new()
+               .D8(0xab)
+               .append_section(Section::new().D8(0xcd))
+               .D8(0xef)
+               .get_contents().unwrap(),
+               &[0xab, 0xcd, 0xef]);
+}
+
+#[test]
+fn section_append_section_labels() {
+    let mut s = Section::new();
+    let l1 = Label::from_const(0x12);
+    let l2 = Label::new();
+    s = s.D8(0xab);
+    {
+        s = s.append_section(Section::new()
+                             .D8(0xcd)
+                             .D8(&l1)
+                             .D8(&l2));
+    }
+    s = s.D8(0xef);
+    l2.set_const(0x34);
+    assert_eq!(s.get_contents().unwrap(),
+               &[0xab, 0xcd, 0x12, 0x34, 0xef]);
+}
+
 #[test]
 fn section_append_repeated() {
     let s = Section::new();
@@ -1036,6 +1080,22 @@ fn section_start_mark() {
     assert_eq!(marked.offset(&start).unwrap(), 10);
 }
 
+#[test]
+fn section_additional_methods_trait() {
+    trait ExtraSection {
+        fn add_a_thing(self) -> Section;
+    }
+
+    impl ExtraSection for Section {
+        fn add_a_thing(self) -> Section {
+            self.B8(0x12).B16(0x3456).B32(0x7890abcd)
+        }
+    }
+
+    assert_eq!(Section::new().D8(0).add_a_thing().D8(1).get_contents().unwrap(),
+               &[0, 0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xcd, 1]);
+}
+
 #[test]
 fn test_simple_labels() {
     let start = Label::new();
